% Author:   Dominik Fahrner
%           Department of Earth Sciences
%           University of Oregon
%           04/11/2022

function FrontalAblation(TWGName)
% Function determines frontal ablation using remote sensing data 
% 
%
% arguments: (input)
% TWGName - GlacierName
% Monthly - Set to 1 if calculation should only be based on monthly data
% WeightedAverage - Set to 1 if calculation should be based on monthly
%                   weighted average data
% GlacierNumber - Used to create figure caption

%%%%%%%%%%%%%%%%%%%%%% Load Data %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Path to TermPicks Terminus Positions
warning('off','all')
AllTerminiPath = strcat('Data/TP/',TWGName); %Qeqertarsuup_Sermia.shp';

% Get Glacier Name
FileName = strsplit(AllTerminiPath,'/');
GlacierName = TWGName;
fprintf('Processing Glacier: %s', GlacierName);
% Specify data and image output folders
% Create Output Folders
outFolderData = 'Results/Data/';
if ~exist(outFolderData, 'dir')
    mkdir(outFolderData)
end

% Load Satellite Image 
[L8Im,RL8] = geotiffread(strcat('Data/Sat_Images/',TWGName,'/PS_3413.tif'));

% Paths to DEMs, Bedrock Topography and surface change rate
DEMFolder=strcat('Data/DEMs/',TWGName,'/');
ArcticDEM =strcat(DEMFolder,dir(fullfile(DEMFolder,'*_20*.tif')).name); %'Data/DEMs/',TWGName,'/',TWGName,'_20*.tif 'Qeqertarsuup_Sermia/Qeqertarsuup_Sermia_20170306.tif';
OlderDEM = strcat(DEMFolder,dir(fullfile(DEMFolder,'*_19*.tif')).name); %'Data/DEMs/Qeqertarsuup_Sermia/Qeqertarsuup_Sermia_1985.tif';
BedDEM = 'Data/BedTopo/BedmachineV4_BedTopography.tif';
dhdt = readmatrix('Data/SurfaceChange/dHdt_Mankoff.csv');

% Discharge Data
PathToFlux =strcat('Data/Flux_w_Error/',TWGName,'.csv');


%Centerline Data
Centerline = strcat('Data/Centerlines/',TWGName, '.shp');
Glacier_Centerline = strcat('Data/Glacier_Centerlines/',TWGName,'.shp');

% Specify Border Color
Ncolor= [99 172 190]/255;
%% Draw Fjord Walls (Upper and Lower)
if exist(strcat(outFolderData,'FjordBoundaries/','UpperFW_',GlacierName,'.csv'))
    fprintf('Boundary files found - Loading files')
    FW = readmatrix(strcat(outFolderData,'FjordBoundaries/','UpperFW_',GlacierName,'.csv'));
    FWl = readmatrix(strcat(outFolderData,'FjordBoundaries/','LowerFW_',GlacierName,'.csv'));
    [UpperPoly,LowerPoly] = CreatePolygon(FW,FWl);
else
    [FW,FWl,UpperPoly,LowerPoly] = drawFjordWalls(L8Im,RL8,GlacierName,outFolderData);
end
%% Download Velocity Composites
% Define StartYear, EndYear, Folder to save the data
% Default is defined as 1985-2018 and "Velocity/NetCDF"
VelocityOutFolder = VelocityDownload();
%% Extract Velocity average in ROI
VelData = VelocityExtractionLine(VelocityOutFolder, Centerline,AllTerminiPath);
%% Average Terminus Positions monthly or Selection of Monthly Terminus Positions
% MonthlytePosFolder = tePosAveraging(FW,FWl,AllTerminiPath);
MonthlyTPSelectFolder = MonthlyTPSelection(AllTerminiPath);
%% Quality Control of Terminus Positions
% Checks TermPicks data for faulty delineations (i.e. too short.long), 
% wrong geolocation and wrong dates. Also splits combined shapefile into
% individual observations
[SplitTerminiFolder,MFW]=TerminiQualityControl(MonthlyTPSelectFolder,VelData,Centerline,UpperPoly,...
    LowerPoly,outFolderData,0.95);

% Split Terminus Positions
TP_files = dir(fullfile(SplitTerminiFolder,'*.shp'));
TP_Names = {TP_files.name};
N = length(TP_Names);
%% Sort Terminus Positions by Date
for i=1:N
    Termini(i)= shaperead(fullfile(SplitTerminiFolder,TP_Names{i}));
end

Afields = fieldnames(Termini);
idx = find(contains(Afields,'DateNum'));
Acell = struct2cell(Termini);
sz = size(Acell);            
Acell = reshape(Acell, sz(1), []);     
Acell = Acell';                        
Acell = sortrows(Acell, idx); 
Acell = reshape(Acell', sz);
SortedTermini = cell2struct(Acell, Afields, 1);
for i=1:length(SortedTermini)
    tePos{i}=SortedTermini(i);
end
clear sz Afields Acell
%% Get closest point on Fjordwall to end/startpoint of Terminus Position
% Checks which way the terminus is drawn (e.g. left to right/right to left 
% or up to down/down to up)and flips them to a consistent drawing direction
% if necessary
f=waitbar(0,'Processing');
for i=1:N
%     tePos{i} = shaperead(fullfile(SplitTerminiFolder,TP_Names{i}));
    if isnan(tePos{i}.X(end))
        tePos{i}.X = tePos{i}.X(:,any(~isnan(tePos{i}.X),1));
        tePos{i}.Y = tePos{i}.Y(:,any(~isnan(tePos{i}.Y),1));
    else
        tePos{i}.X = tePos{i}.X;
        tePos{i}.Y  =tePos{i}.Y;     
    end
    if ~isnan(tePos{i}.X(end))
        TerminusEndPoint = tePos{i}.X(end);
        TerminusEndPoint(:,2) =tePos{i}.Y(end);
    else
        TerminusEndPoint = tePos{i}.X(end-1);
        TerminusEndPoint(:,2) =tePos{i}.Y(end-1);
    end
    TerminusStartPoint = tePos{i}.X(1,1);
    TerminusStartPoint(:,2) = tePos{i}.Y(1,1);
    %Check distance of Terminus EndPoints to either fjord wall to see if
    % they are consistently draw N/S, S/N, or E/W, W/E
    [~,DEndToUpper,~] = distance2curve(FW,TerminusEndPoint);
    [~,DEndToLower,~] = distance2curve(FWl,TerminusEndPoint);

    if min(DEndToLower)<min(DEndToUpper) 
        tePos{i}.X = fliplr([tePos{i}.X]);
        tePos{i}.Y = fliplr([tePos{i}.Y]);
    else
        tePos{i}.X =tePos{i}.X ;
        tePos{i}.Y =tePos{i}.Y;
    end

    NewTerminusEndPoint = tePos{i}.X(end);
    NewTerminusEndPoint(:,2) =tePos{i}.Y(end);
    NewTerminusStartPoint = tePos{i}.X(1);
    NewTerminusStartPoint(:,2) =tePos{i}.Y(1);

waitbar(i/N, f, sprintf('Processing drawing direction: %d%%',floor(i/N*100)));
end
close(f)
clear DEndToUpper DEndToLower DStartToUpper DStartToLower TerminusEndPoint TerminusStartPoint
clear NewTerminusEndPoint NewTerminusStartPoint
%%
% Check where Margin Endposition is located in relation to upper/lower fjord wall boundary 
% Extrapolates terminus position start/endpoint to fjord wall boundary if it 
% doesn't intersect with the fjord wall or crops it to the point of
% intersection
f= waitbar(0,'Processing');
for i=1:N
    % Check if terminus position intersects fjord walls more than once
    [xi,yi,] = polyxpoly(tePos{i}.X,tePos{i}.Y,FW(:,1),FW(:,2));
    if size(xi)>1
        EndCoords(:,1) = xi(1);
        EndCoords(:,2) = yi(1);
    else
        EndCoords(:,1) = tePos{i}.X(end);
        EndCoords(:,2) = tePos{i}.Y(end);
    end

    [InUpper,OnUpper,Number] = InPolygonCheck(tePos{i},UpperPoly,0,1);
% Crop to intersection with fjord wall point if terminus endpoint is
% located above fjord wall
    if InUpper==1 
        disp('Above Upper Fjord Wall');
        PointsAbove = Number;
        if PointsAbove>0
            [InterX,InterY]= polyxpoly(tePos{i}.X,tePos{i}.Y,FW(:,1),FW(:,2));
            StorageEndPoint = [InterX(1),InterY(1)];
            [~,IntIdxUp] = pdist2(FW, StorageEndPoint,'euclidean','Smallest',1);
            NewFWEndPointUp= [FW(IntIdxUp,1),FW(IntIdxUp,2)];
            tePos{i}.X = tePos{i}.X(1:end-PointsAbove);
            tePos{i}.Y = tePos{i}.Y(1:end-PointsAbove);
            tePos{i}.X(end) = NewFWEndPointUp(:,1);
            tePos{i}.Y(end) = NewFWEndPointUp(:,2);
        end
        if InUpper==1 && OnUpper==1 && PointsAbove>0
            [InterXDouble,InterYDouble]= polyxpoly(tePos{i}.X,tePos{i}.Y,FW(:,1),FW(:,2));
            StorageEndPoint = [InterXDouble(1),InterYDouble(1)];
            [~,IntIdxUp] = pdist2(FW, StorageEndPoint,'euclidean','Smallest',1);
            NewFWEndPointUp= [FW(IntIdxUp,1),FW(IntIdxUp,2)];
            tePos{i}.X = tePos{i}.X(1:end-PointsAbove);
            tePos{i}.Y = tePos{i}.Y(1:end-PointsAbove);
            tePos{i}.X(end) = NewFWEndPointUp(:,1);
            tePos{i}.Y(end) = NewFWEndPointUp(:,2);
        end
% Extrapolate to intersection with fjord walls if terminus endpoint is 
% located below fjord wall
    elseif InUpper==0 
        disp('Below Upper Fjord Wall');
        [~,InterIdx] = pdist2(FW, EndCoords,'euclidean','Smallest',1);
        NewFWEndPointUp = [FW(InterIdx,1),FW(InterIdx,2)];
        tePos{i}.X(end) = NewFWEndPointUp(:,1);
        tePos{i}.Y(end) = NewFWEndPointUp(:,2);
    elseif OnUpper==1
% Do nothing if terminus endpoint is already on fjord wall
        disp('On Upper Fjord Wall');
        tePos{i}.X(end) = tePos{i}.X(end);
        tePos{i}.Y(end) = tePos{i}.Y(end);
    end
   
clear  StorageEndPoint InterX InterY InterXDouble InterYDouble InterIdx InterSdist 
clear xi yi NewFWEndPointUp InUpper OnUpper PointsAbove EndCoords IntIdxUp InterS Int 
clear TempEndP ClosestIntIdx EndP

    % Repeat above steps for lower fjord wall and terminus start points
    StartCoords(:,1) = tePos{i}.X(1);
    StartCoords(:,2) = tePos{i}.Y(1);
    [xiD,~,LSeg] = polyxpoly(tePos{i}.X,tePos{i}.Y,FWl(:,1),FWl(:,2));
    if length(xiD)>1
        tePos{i}.X = tePos{i}.X(max(LSeg)-1:end);
        tePos{i}.Y = tePos{i}.Y(max(LSeg)-1:end);
    end
    [InLower,OnLower,PointsBelow] = InPolygonCheck(tePos{i},LowerPoly,1,0);
    if InLower==0 
        [~,InterIdx] = pdist2(FWl, StartCoords,'euclidean','Smallest',1);
        NewXYDown = zeros(length(tePos{i}.X)+1,2);
        NewXYDown(1,1) = FWl(InterIdx,1);
        NewXYDown(1,2) = FWl(InterIdx,2);
        NewXYDown(2:end,1) = tePos{i}.X;
        NewXYDown(2:end,2) = tePos{i}.Y;
        tePos{i}.X = NewXYDown(:,1);
        tePos{i}.Y = NewXYDown(:,2);
    elseif InLower==1 
        % Check if terminus position intersects fjord walls more than once and if so, crop to first intersection point
        if PointsBelow>0
           [InterXLower,InterYLower]= polyxpoly(tePos{i}.X,tePos{i}.Y,FWl(:,1),FWl(:,2)); 
            tePos{i}.X = tePos{i}.X(1+PointsBelow:end);
            tePos{i}.Y = tePos{i}.Y(1+PointsBelow:end);
            NewXYDown = zeros(length(tePos{i}.X)+1,2);
            NewXYDown(1,1) = InterXLower(1);
            NewXYDown(1,2) = InterYLower(1);
            NewXYDown(2:end,1) = tePos{i}.X;
            NewXYDown(2:end,2) = tePos{i}.Y;
            tePos{i}.X = NewXYDown(:,1); 
            tePos{i}.Y = NewXYDown(:,2);
        end
    elseif InLower==1 && OnLower==1 && PointsBelow>0
            [InterXLower,InterYLower]= polyxpoly(tePos{i}.X,tePos{i}.Y,FWl(:,1),FWl(:,2)); 
            tePos{i}.X = tePos{i}.X(1+PointsBelow:end);
            tePos{i}.Y = tePos{i}.Y(1+PointsBelow:end);
            NewXYDown = zeros(length(tePos{i}.X)+1,2);
            NewXYDown(1,1) = InterXLower(1);
            NewXYDown(1,2) = InterYLower(1);
            NewXYDown(2:end,1) = tePos{i}.X;
            NewXYDown(2:end,2) = tePos{i}.Y;
            tePos{i}.X = NewXYDown(:,1);
            tePos{i}.Y = NewXYDown(:,2);
    elseif OnLower==1
        tePos{i}.X(1) = tePos{i}.X(1);
        tePos{i}.Y(1) = tePos{i}.Y(1);
    end
       waitbar(i/N, f, sprintf('Processing terminus positions: %d%%',floor(i/N*100)));
end
close(f)
clear NewXYDown InterXLower InterYLower InterIdx  
clear OnLower InLower PointsBelow StartCoords xiD LSeg
%% Check Extrapolation
% % Check if length of extrapolation exceeds actual terminus delineation
for i=1:length(tePos)
    ExCoordsStart{i} =[tePos{i}.X(1:2)';tePos{i}.Y(1:2)'];
    ExCoordsEnd{i} = [tePos{i}.X(end-1:end)';tePos{i}.Y(end-1:end)'];
    DelinCoords{i} = [tePos{i}.X';tePos{i}.Y'];
    dDelin{i} = diff(DelinCoords{i}); 
    
    DelinLen(i,1) = sum(sqrt(sum(dDelin{i}.*dDelin{i},2)));
    ExTotalLen(i,1)= norm(ExCoordsStart{i}(1:2,1)-ExCoordsStart{i}(1:2,2)) + norm(ExCoordsEnd{i}(1:2,1)-ExCoordsEnd{i}(1:2,2));
    if (ExTotalLen(i,1)<=DelinLen(i,1))==1 
        tePosTemp{i}=tePos{i};
    else
        tePosTemp{i}=[];
        disp('Extrapolation > Delineation')
    end
end
tePos=tePosTemp(~cellfun('isempty',tePosTemp));
N=length(tePos);
clear tePosTemp dTerminus TerminusLength dExtrapolation ExtrapolationLength

%% Define Boundary upstream of last Terminus Position
% as the most retreated position of the terminus might not be the most
% retreated across the terminus, an arbitrary lower boundary should be
% drawn upstream of the most retreated terminus position. The discrepancy
% will be subtracted later on. 
if exist(strcat(outFolderData,'UpstreamBoundary/',GlacierName,'.csv'))
    disp('Upstream Boundary files found - Loading files')
    BLN = readmatrix(strcat(outFolderData,'UpstreamBoundary/',GlacierName,'.csv'));
else
    BLN =LowerBoxBoundary(L8Im,RL8, FW,FWl,tePos,GlacierName,outFolderData);
end
%% Crop the fjord wall boundaries to the specific terminus boundaries
% Needed to create a polygon in the next step. 
for i=1:N
    FWUpperStart = [tePos{i}.X(end),tePos{i}.Y(end)];
    FWUpperEnd =[BLN(1,1),BLN(1,2)];
    FWLowerStart=[tePos{i}.X(1),tePos{i}.Y(1)];
    FWLowerEnd= [BLN(end,1),BLN(end,2)];

    UpperStartIdx = find(FW==round(FWUpperStart,25));
    if isempty(UpperStartIdx)==1
        UpperStartIdx = dsearchn(FW, FWUpperStart);
    end
    [EndXUp,EndYUp] = polyxpoly(FW(:,1),FW(:,2),FWUpperEnd(:,1),FWUpperEnd(:,2));
    StorageEndPointUp = [EndXUp,EndYUp];
    [~,UpperEndIdx] = pdist2(FW, StorageEndPointUp,'euclidean','Smallest',1);
    
    IndFW{i}.X = FW(UpperStartIdx:UpperEndIdx,1);
    IndFW{i}.Y = FW(UpperStartIdx:UpperEndIdx,2);
    
    %%%%%% Same for lower boundary %%%%%
    LowerStartIdx = find(FWl==FWLowerStart);
    if isempty(LowerStartIdx)==1 
        LowerStartIdx = dsearchn(FWl, FWLowerStart);
        StartIdxLoKeep=1;
    end
    [EndXLo,EndYLo] = polyxpoly(FWl(:,1),FWl(:,2),FWLowerEnd(:,1),FWLowerEnd(:,2));

    StorageStartPointLo = [EndXLo,EndYLo];
    [~,LowerEndIdx] = pdist2(FWl, StorageStartPointLo,'euclidean','Smallest',1);

    IndFWl{i}.X = FWl(LowerStartIdx:LowerEndIdx,1);
    IndFWl{i}.Y = FWl(LowerStartIdx:LowerEndIdx,2);
    IndFWl{i}.X(1)=FWLowerStart(:,1);
    IndFWl{i}.Y(1)=FWLowerStart(:,2);
end
for i=1:length(tePos)
    OutShp(i) = tePos{i};
end
if ~exist(strcat(outFolderData,'Final_Shapefiles/'))
    mkdir(strcat(outFolderData,'Final_Shapefiles/'))
end
shapewrite(OutShp,strcat(outFolderData,'Final_Shapefiles/',GlacierName,'.shp'));

clear LowerStartIdx StorageEndPointUp StorageStartPointLo UpperStartIdx
clear UpperEndIdx LowerEndIdx EndXLo EndYlo EndXUp EndXLo OutShp StartIdxLoKeep
clear FWLowerEnd FWUpperStart FWUpperEnd FWLowerEnd EndYLo EndYUp
%% Concatenate Boundaries, create polygon and calculate area
disp('Concatenating boundaries')
for i=1:N
    TerminusPosition = tePos{i}.X; 
    TerminusPosition(:,2)=tePos{i}.Y;
    UpperBoundary=IndFW{i}.X; 
    UpperBoundary(:,2)=IndFW{i}.Y;
    LowerBoundary=IndFWl{i}.X; 
    LowerBoundary(:,2)=IndFWl{i}.Y;

    Poly{i}= polyshape([TerminusPosition; UpperBoundary;BLN;flipud(LowerBoundary)]);
    PolygonGeometry(i,1) = datenum(tePos{i}.Date); % Observation Date
    PolygonGeometry(i,2) = area(Poly{i})/1e6;      % Polygon Area [km^2]
end
%% Get Surface Elevation and Ice thickness
disp('Calculating surface elevation change')
[IceThicknessData] = SurfaceElevationChange(ArcticDEM,OlderDEM,BedDEM,dhdt,Poly,PolygonGeometry(:,1));
%% Get mean ice thickness in polygon and calculate volume
for i=1:N
    ROImask{i} = inpolygon(IceThicknessData{i}.X,IceThicknessData{i}.Y,Poly{i}.Vertices(:,1),Poly{i}.Vertices(:,2)); 
    IceThickness{i}.X = IceThicknessData{i}.X.*ROImask{i};
    IceThickness{i}.Y = IceThicknessData{i}.Y.*ROImask{i};
    IceThickness{i}.H = IceThicknessData{i}.H.*ROImask{i};
    IceThickness{i}.X(IceThickness{i}.X==0)=NaN;
    IceThickness{i}.Y(IceThickness{i}.Y==0)=NaN;
    IceThickness{i}.H(IceThickness{i}.H==0)=NaN;
    MeanH(i,1) = nanmean(IceThickness{i}.H,'all');
end

VolumeTS(:,1) = PolygonGeometry(:,1); % Observation Dates
VolumeTS(:,2) = PolygonGeometry(:,2); % Polygon Area [km^2]
VolumeTS(:,3) = MeanH./1000; % Mean Ice Thickness [km]
VolumeTS(:,4) = VolumeTS(:,2).*VolumeTS(:,3); % Polygon Volume [km^3]
VolumeTS(:,5) = VolumeTS(:,4)*0.917; % Mass 
save([outFolderData,'/VOL_Mat/',GlacierName,'_Volume_timeseriesAV.mat'],'VolumeTS','-v7.3');
%% Determine terminus position change along the centerline
TPC = CenterlineMethod(tePos,Centerline);
%% Calculate uncertainties
TMC_Error = TMC_Error_calc(TPC,PathToFlux,MFW,MeanH,Glacier_Centerline,'three-monthly');
save([outFolderData,'/TMC_ErrorMat/',GlacierName,'TMC_Error.mat'],'TMC_Error','-v7.3');

%% Get ice flow velocities
VData = readmatrix(VelData);
VData(:,3) = ones(length(VData),1);
VData(:,1) = datenum(datetime(VData(:,1),VData(:,3),VData(:,3),'Format','yyyy-MM-dd'));

%% Plot results
plotFrontalAblation(PathToFlux,VolumeTS,TPC,VData,tePos,TMC_Error,...
    GlacierName,'three-monthly',outFolderData)

end