function [ITData]= SurfaceElevationChange(PathToArcticDEM,PathToAeroDEM,...
    PathToBedrockTopo,dhdt,Poly,Dates)
% Function determines Surface elevation change per year based on surface 
% elvation change rate determined by Khan et al. (2017) for the years 
% 1995 - 2015. Prior and after these dates, surface elevation change is 
% assumed to be linear and calculated using two an ArcticDEM (Porter et al.
% , 2018)and a DEM derived from orthophotos (Korsgaard et al., 2016). The 
% ice thickness is calcualate as difference between the adjusted surface 
% elvation  and the bedrock % topography (automatically cropped to DEM 
% extent)for each available date of terminus positions. 
%
% arguments: (input)
% PathToArcticDEM     - Path to directory of latest DEM. DEM has to be in Polar 
%                       Stereographic projection (EPSG:3413).
% PathToAeroDEM       - Path to directory of oldest available DEM.DEM has to be 
%                       in Polar Stereographic projection (EPSG:3413).
% PathToBedrockTopo   - Path the Bedmachine v4 data set.
% dhdt                - Surface elevation change rates determined by Khan
%                       et al. (2017) [m/yr
% Poly                - Polygon geometry used to determine which DEM points 
%                       lie within Area of interest
% Dates               - dd/mm/yyyy of digitised termini
%
% Input raster data is automatically downsampled to the resolution of the
% bedrock topography data (150 m)
%
% arguments: (output)
% DEM.X - X coordinates of corresponding to caluclated ice thickness values
% DEM.Y - Y coordinates of corresponding to caluclated ice thickness values     
% DEM.H - Ice thickness values calculated using Mankoff elevation change 
%         data from Mankoff et al.(2019) and assuming linear change in surface
%         elevation for observation prior to 1995 and after 2015. Annual 
%         change of surface elevation is determined by  differencing the
%         two DEMs and dividing values by number of years between both. 
%         Surface elevation is heightend by adding the annual elevation 
%         change multiplied by the elapsed time between terminus 
%         observations for years earlier than the creation date of the
%         oldest DEM. Surface elevation is lowered by subtracting the annual 
%         elevation change multiplied by the elapsed time between terminus 
%         observations for years later than the creation date of the
%         oldest DEM. 
%
%
% NOTE: These calculations assume linear changes in surface elevation before
% 1995 and after 2015 and therefore do not account for annual dynamic
% variability



[ArcticDEM,RArctic] = geotiffread(PathToArcticDEM);
[AeroDEM,RAero] = geotiffread(PathToAeroDEM);
[Bed,Rbed] = geotiffread(PathToBedrockTopo);
AeroDEM(AeroDEM>20000)=NaN;
AeroDEM(AeroDEM<-20000)=NaN;
[~,FileName,~]=fileparts(PathToAeroDEM);
Bedinfo = geotiffinfo(PathToBedrockTopo);

% Resampling DEMs to Bedrock topography resolution
[ResampArcticDEM,RArcticResamp] = mapresize(ArcticDEM,RArctic,(RArctic.CellExtentInWorldX/Bedinfo.PixelScale(1)),'nearest');
[ResampAeroDEM,RAeroResamp] = mapresize(AeroDEM,RAero,(RAero.CellExtentInWorldX/Bedinfo.PixelScale(1)));

sizeArcticDEM = size(ResampArcticDEM);
sizeAeroDEM = size(ResampAeroDEM);
if (sizeArcticDEM<sizeAeroDEM)==1
    % Cropping Older DEM & Bedrock Topography to extent of newer DEM
    xlimits = [RArcticResamp.XWorldLimits(1),RArcticResamp.XWorldLimits(2)];
    ylimits = [RArcticResamp.YWorldLimits(1),RArcticResamp.YWorldLimits(2)];
    
    [CropAeroDEM,RAeroCrop] = mapcrop(ResampAeroDEM,RAeroResamp,xlimits,ylimits);
    [CropBed,RbedCrop] = mapcrop(Bed,Rbed,xlimits,ylimits);
else
    xlimits = [RAeroResamp.XWorldLimits(1),RAeroResamp.XWorldLimits(2)];
    ylimits = [RAeroResamp.YWorldLimits(1),RAeroResamp.YWorldLimits(2)];
    
    [CropArcticDEM,RCropArcticDEM] = mapcrop(ResampArcticDEM,RArcticResamp,xlimits,ylimits);
    [CropBed,RbedCrop] = mapcrop(Bed,Rbed,xlimits,ylimits);
end
%%% Check if all Images have the same x/y size
if exist('CropAeroDEM','var')
    if isequal(CropBed,CropAeroDEM,ResampArcticDEM)==0 % Check if all files have the same size and if not: 
        sizeDifferenceDEMs = size(ResampArcticDEM) - size(CropAeroDEM); % Get size difference for DEMs
        if (sum(sizeDifferenceDEMs~=0))>0% if they are the same size
         % if there is a size difference between the DEMs, make them the same size and Bed Topo
                SizeMinDEM = min(size(CropAeroDEM), size(ResampArcticDEM));
                CropAeroDEM = CropAeroDEM(1:SizeMinDEM(1),1:SizeMinDEM(2));
                ResampArcticDEM = ResampArcticDEM(1:SizeMinDEM(1),1:SizeMinDEM(2));
        end
    end
        sizeDifferenceDEM_Bed = size(CropAeroDEM) - size(CropBed); % Check size difference between Bed and DEM
        if (sizeDifferenceDEM_Bed~=0) ==1 % if there is a size difference
            SizeMin = min(size(CropAeroDEM), size(CropBed));
            CropBed = CropBed(1:SizeMin(1),1:SizeMin(2));
        end
        RbedCrop.RasterSize = size(CropBed);
        RArcticResamp.RasterSize = size(ResampArcticDEM);
        RAeroCrop.RasterSize = size(CropAeroDEM);

end
if exist('CropArcticDEM','var')
    if isequal(CropBed,ResampAeroDEM,CropArcticDEM)==0 % Check if all files have the same size and if not: 
        sizeDifferenceDEMs = size(ResampAeroDEM) - size(CropArcticDEM); % Get size difference for DEMs
        if (sizeDifferenceDEMs~=0)>0% if they are the same size
         % if there is a size difference between the DEMs, make them the same size and Bed Topo
                SizeMinDEM = min(size(CropArcticDEM), size(ResampAeroDEM));
                CropArcticDEM = CropArcticDEM(1:SizeMinDEM(1),1:SizeMinDEM(2));
                ResampAeroDEM = ResampAeroDEM(1:SizeMinDEM(1),1:SizeMinDEM(2));
        end
    end
    sizeDifferenceDEM_Bed = size(CropArcticDEM) - size(CropBed); % Check size difference between Bed and DEM
    if (sizeDifferenceDEM_Bed~=0) ==1 % if there is a size difference
        SizeMin = min(size(CropArcticDEM), size(CropBed));
        CropBed = CropBed(1:SizeMin(1),1:SizeMin(2));
    end
    RbedCrop.RasterSize = size(CropBed);
    RCropArcticDEM.RasterSize = size(CropArcticDEM);
    RAeroResamp.RasterSize = size(ResampAeroDEM);
end

% Get BedTopo raster coordinates
lon0 = RbedCrop.XWorldLimits(1);   % Top Left longitude coordinate from .vrt files
lat0 = RbedCrop.YWorldLimits(1); 
dlon = RbedCrop.CellExtentInWorldX;
dlat = RbedCrop.CellExtentInWorldY;% stepping in lon/lat
lon = lon0: dlon:RbedCrop.XWorldLimits(2);  % Create #vector of longitudes
lat = lat0: dlat:RbedCrop.YWorldLimits(2);

clear SizeMin SizeMinDEM sizeArcticDEM sizeAeroDEM sizeDifferenceDEM_Bed sizeDifferenceDEMs Bed Rbed AeroDEM RAero
clear ArcticDEM RArctic  Bedinfo 

%%  
% Preallocate cell arrays
% DataIdx = cell(length(Poly),1);
DateIdx = cell(length(Poly),1);
SCRate = cell(length(Poly),1);
Change = cell(length(Poly),1);
NewSurfaceHeight =cell(length(Poly),1);

% Find acquisition year of earliest and latest DEM and calculate yearly change

k = strfind(PathToAeroDEM,'19');
if ~isempty(k)
    EarliestDEM = str2double(PathToAeroDEM(k:k+3));
else
    k= strfind(PathToAeroDEM,'20');
    EarliestDEM = str2double(PathToAeroDEM(k:k+3));
end
l = strfind(PathToArcticDEM,'20');
ArcticDEMYear = str2double(PathToArcticDEM(l:l+3));

if exist('CropAeroDEM','var')
    DiffDEM = (ResampArcticDEM) - (CropAeroDEM); 
else
    DiffDEM =(CropArcticDEM) - (ResampAeroDEM);
end
DiffDEM(DiffDEM<-1e+38)=NaN;
YearlyDiff = DiffDEM./(ArcticDEMYear-EarliestDEM);
yearlyChange = nanmean(YearlyDiff(:));

% Calculate mean rate of surface elevation change in AOI from Khan et al.,
% (2016) used in Mankoff et al., (2019)
dhdt_data = dhdt(1:end,3:end);
dhdt_location=dhdt(:,1:2);
Mankoff_dates = dhdt(1,3:end);
ObsYears=sortrows(year(Dates));

%Preallocate cell arrays
DataIdx = cell(length(ObsYears),1);
DateIdx = cell(length(ObsYears),1);
SCRate = cell(length(ObsYears),1);

% Convert from Lat/Lon to polarstereographic (ESPG:3413)
for i=1:length(Poly)
    [x,y] = polarstereo_fwd(dhdt_location(:,1),dhdt_location(:,2),6378273,0.081816153,70,-45);
    DataInPolygon{i}=inpolygon(x,y,Poly{i}.Vertices(:,1),Poly{i}.Vertices(:,2));
    DataIdx{i} = find(DataInPolygon{i}==1);
end
for i=1:length(ObsYears)
        DateIdx{i}=find(Mankoff_dates==ObsYears(i));
        if ~isempty(DateIdx{i})
            SCRate{i} = nanmean(dhdt_data(DataIdx{i},DateIdx{i}));     
        else
            SCRate{i}=[];
        end
end
SCNaNCheck = [SCRate{:}]'; % Change to vector for NaN Check
%%
lon02 = RArcticResamp.XWorldLimits(1);   % Top Left longitude coordinate from .vrt files
lat02 = RArcticResamp.YWorldLimits(2); 
dlon2 = (RArcticResamp.CellExtentInWorldX+ RbedCrop.CellExtentInWorldX)/2;
dlat2 = (RArcticResamp.CellExtentInWorldY+ RbedCrop.CellExtentInWorldY)/2;% stepping in lon/lat
lon2 = lon02: dlon2:RArcticResamp.XWorldLimits(2);  % Create #vector of longitudes
lat2 = lat02: -dlat2:RArcticResamp.YWorldLimits(1);

[xDEMfinal,yDEMfinal] = meshgrid(lon2,lat2);

%%
% If no Mankoff/Khan data is available subtract/add surface elevation change
% form 1985 DEM based on timedelta between Terminus position observations 
% and get Ice Thickness (H) by subtracting surface elvation from bedrock elevation

% Preallocate matrices and cell arrays
% SingleDateIdx =cell(length(ObsYears),1);
% SCRatePerYear = zeros(length(Mankoff_dates),2);
% MankoffScRate = zeros(length(ObsYears),1);
% SurfaceChangeRate = zeros(length(ObsYears),1);
% DEMScRate = zeros(length(ObsYears),1);

 for i=1:length(Dates)
    if all(isnan(SCNaNCheck))==1
        disp('No Mankoff Data available)')
         if exist('CropAeroDEM','var') % Check which DEM has been cropped
                if ObsYears(i)<ArcticDEMYear
                    IceThicknessData{i}.X = xDEMfinal;
                    IceThicknessData{i}.Y = yDEMfinal;
                    timeDelta = ArcticDEMYear-ObsYears(i); 
                    NewSurfaceHeight{i} = ResampArcticDEM +(timeDelta*abs(yearlyChange)); %
                    IceThicknessData{i}.H = NewSurfaceHeight{i} - CropBed; 
                elseif ObsYears(i)==ArcticDEMYear
                    IceThicknessData{i}.X = xDEMfinal;
                    IceThicknessData{i}.Y = yDEMfinal;
                    NewSurfaceHeight{i} = ResampArcticDEM; %
                    IceThicknessData{i}.H = NewSurfaceHeight{i} - CropBed; 
                elseif ObsYears(i)>ArcticDEMYear
                    IceThicknessData{i}.X = xDEMfinal;
                    IceThicknessData{i}.Y = yDEMfinal;
                    timeDelta = ObsYears(i)-ArcticDEMYear;
                    NewSurfaceHeight{i} = ResampArcticDEM -(timeDelta*abs(yearlyChange));
                    IceThicknessData{i}.H = NewSurfaceHeight{i} - CropBed;
                end
                
         else
                if ObsYears(i)<ArcticDEMYear
                    IceThicknessData{i}.X = xDEMfinal;
                    IceThicknessData{i}.Y = yDEMfinal;
                    timeDelta = ArcticDEMYear - ObsYears(i); 
                    NewSurfaceHeight{i} = CropArcticDEM +(timeDelta*abs(yearlyChange)) ;
                    IceThicknessData{i}.H = NewSurfaceHeight{i} - CropBed; 
                elseif ObsYears(i)==ArcticDEMYear
                    IceThicknessData{i}.X = xDEMfinal;
                    IceThicknessData{i}.Y = yDEMfinal;
                    NewSurfaceHeight{i} = CropArcticDEM;
                    IceThicknessData{i}.H = NewSurfaceHeight{i} - CropBed; 
                elseif ObsYears(i)>ArcticDEMYear
                    IceThicknessData{i}.X = xDEMfinal;
                    IceThicknessData{i}.Y = yDEMfinal;
                    timeDelta = ObsYears(i)-ArcticDEMYear; 
                    NewSurfaceHeight{i} = CropArcticDEM -(timeDelta*abs(yearlyChange));
                    IceThicknessData{i}.H = NewSurfaceHeight{i} - CropBed;
                end
         end
     DEMstruct = IceThicknessData;
    else
        % If Khan/Mankoff data is available, calculate the rate of change per year
        % within the AOI. If the observation is outside the Khan/Mankoff temporal
        % bounds, the mean annual change rate is added to the latest 
        
        
        if all(isnan(SCNaNCheck))==0
            disp('Mankoff Data Available')
            SCRateTemp(:,1) = ObsYears;
            A = find(~cellfun(@isempty,SCRate),1);
            O = find(~cellfun(@isempty,SCRate),1,'last');
            SCRateTemp(A:O,2) = [SCRate{:}];
            [B,~,gi] = unique(SCRateTemp(:,1),'rows','stable'); 
            B(:,2) = accumarray(gi(:), SCRateTemp(:,2), [], @nanmean);
            ZeroIdx = find(B(:,2)==0);
            B(ZeroIdx,:)=[];
            SCRatePerYear =B;
        end  
        SCRatePerYear=SCRatePerYear(all(~isnan(SCRatePerYear),2),:);
        clear A O B SCRateTemp gi ZeroIdx
        %%    % Sum Mankoff change rates for the years between ArcticDEM acquisition year and observation. 
        
        for i=1:length(ObsYears)
            if ArcticDEMYear>=min(Mankoff_dates)&& ArcticDEMYear<=max(Mankoff_dates)&& max(SCRatePerYear(:,1))>=ArcticDEMYear
                MankoffIdxStart= find(SCRatePerYear(:,1)==ArcticDEMYear);
                MankoffIdxEnd=   find(SCRatePerYear(:,1)==ObsYears(i));
                if MankoffIdxStart>MankoffIdxEnd
                   MankoffScRate(i,1) = abs(sum(SCRatePerYear(MankoffIdxEnd:MankoffIdxStart,2)));
                else
                   MankoffScRate(i,1) = abs(sum(SCRatePerYear(MankoffIdxStart:MankoffIdxEnd,2)));
                end
            else
                MankoffIdxStart= find(SCRatePerYear(:,1)==max(SCRatePerYear(:,1)));
                MankoffIdxEnd=   find(SCRatePerYear(:,1)==ObsYears(i));
                if MankoffIdxStart>MankoffIdxEnd
                   MankoffScRate(i,1) = abs(sum(SCRatePerYear(MankoffIdxEnd:MankoffIdxStart,2)));
                else
                   MankoffScRate(i,1) = abs(sum(SCRatePerYear(MankoffIdxStart:MankoffIdxEnd,2)));
                end
            end
        end
             %% Create Surface change rates for each observation
        for i=1:length(ObsYears)
            if ObsYears(i)>=min(Mankoff_dates)&& ObsYears(i)<=max(Mankoff_dates) % if inside Mankoff range
                 SurfaceChangeRate(i) = MankoffScRate(i); % SCrate taken from Mankoff
            else
                if ObsYears(i)<min(Mankoff_dates) % If observation year prior to Mankoff range
                    DEMtimeDelta =abs(min(Mankoff_dates)-ObsYears(i)); % Get timedelta between ArcticDEM and observation year
                    DEMScRate(i,1) = DEMtimeDelta*abs(yearlyChange); % Get SC rate from DEMs
                    SurfaceChangeRate(i)=MankoffScRate(find(MankoffScRate~=0,1,'first')) + DEMScRate(i); 
                end
                if ObsYears(i)>Mankoff_dates(end)
                    DEMtimeDelta =abs(ObsYears(i)-ArcticDEMYear);
                    DEMScRate(i,1) = DEMtimeDelta*abs(yearlyChange);
                    SurfaceChangeRate(i)=MankoffScRate(find(MankoffScRate~=0,1,'last')) + DEMScRate(i);
                end
            end
        end
        %%
        for i=1:length(Dates)
              if exist('CropAeroDEM','var')
                    if ObsYears(i)<ArcticDEMYear
                        IceThicknessData{i}.X = xDEMfinal;
                        IceThicknessData{i}.Y = yDEMfinal;
                        NewSurfaceHeight{i} = ResampArcticDEM +SurfaceChangeRate(i); %
                        IceThicknessData{i}.H = NewSurfaceHeight{i} - CropBed; 
                    elseif ObsYears(i)==ArcticDEMYear
                        IceThicknessData{i}.X = xDEMfinal;
                        IceThicknessData{i}.Y = yDEMfinal;
                        NewSurfaceHeight{i} = ResampArcticDEM; %
                        IceThicknessData{i}.H = NewSurfaceHeight{i} - CropBed; 
                    elseif ObsYears(i)>ArcticDEMYear
                        IceThicknessData{i}.X = xDEMfinal;
                        IceThicknessData{i}.Y = yDEMfinal;
                        NewSurfaceHeight{i} = ResampArcticDEM -SurfaceChangeRate(i);
                        IceThicknessData{i}.H = NewSurfaceHeight{i} - CropBed;
                    end
              else
                    if ObsYears(i)<ArcticDEMYear
                        IceThicknessData{i}.X = xDEMfinal;
                        IceThicknessData{i}.Y = yDEMfinal;
                        NewSurfaceHeight{i} = CropArcticDEM +SurfaceChangeRate(i);
                        IceThicknessData{i}.H = NewSurfaceHeight{i} - CropBed; 
                    elseif ObsYears(i)==ArcticDEMYear
                        IceThicknessData{i}.X = xDEMfinal;
                        IceThicknessData{i}.Y = yDEMfinal;
                        NewSurfaceHeight{i} = CropArcticDEM; %
                        IceThicknessData{i}.H = NewSurfaceHeight{i} - CropBed; 
                    elseif ObsYears(i)>ArcticDEMYear
                        IceThicknessData{i}.X = xDEMfinal;
                        IceThicknessData{i}.Y = yDEMfinal;
                        NewSurfaceHeight{i} = CropArcticDEM -SurfaceChangeRate(i);
                        IceThicknessData{i}.H = NewSurfaceHeight{i} - CropBed;
                    end
            end
        end
    end
 end
        for i=1:length(IceThicknessData)
            CheckData(i,1) = nanmean(NewSurfaceHeight{i}(:));
        end
        
        IceThicknessData{i}.X(IceThicknessData{i}.X==0)=NaN;
        IceThicknessData{i}.Y(IceThicknessData{i}.Y==0)=NaN;
        IceThicknessData{i}.H(IceThicknessData{i}.H==0)=NaN;
        ITData = IceThicknessData;
end
