function Error = TMC_Error_calc(TPC,Discharge,MFW, MeanH,Glacier_Centerline,Seasonality)
% Calculates uncertainties using the shoelace approach using the errors of
% the input datasets. The uncertainties are time averaged over the same time
% period as frontal ablation (monthly, three-monthly, annual) depending on
% the user input
% arguments: (input)
% TPC                   - Terminus change along the centerline between subsequent
%                         observations
% Discharge             - Path to solid ice discharge data
% W                     - Mean fjord width
% MeanH                 - Mean ice thickness
% Glacier_Centerline    - Centerline for the respective glacier
% Seasonality           - Time period over which uncertainties are averaged 
%
% arguments: (output)
% Error.deltaL          - Terminus delineation Error constant (30 metres) [km]
% Error.Rho             - Ice density constant (0.917) [kg/km3]
% Error.T               - Time frame over which frontal ablation is
%                         calculated [d]
% Error.W               - Mean fjord width [km]
% Error.MeanBTError     - Mean bedrock topography error along a centerline in meters
%                         [km]
% Error.MeanHmax        - Maximum mean ice thickness (along centerline) over time period [km]
% Error.deltaHmax       - Maximum ice thickness error using uncertainties from ArcticDEM (0.1
%                         meters), AeroDEM (6 meters), and BTError [km]
% Error.L               - Time averaged terminus change:
%                         tbounds - boundaries of time intervals on which output data defined
%                         t       - midpoint of time intervals on which output data defined
%                         L       - Interpoldated time avearged terminus
%                                     change [km]
%                         L0      - Time averaged terminus change without
%                                     inerpolated values [km]
% Error.D_Error         - Time averaged discharge error:
%                         tbounds - boundaries of time intervals on which output data defined
%                         t       - midpoint of time intervals on which output data defined
%                         DE      - Interpolated time averaged discharge
%                                     error [GT/timeperiod]
%                         DE0     - Time averaged discharge error without
%                                     interpolated values [GT/timeperiod]
% Error.TMC_Error       - Terminus change error [GT/timeperiod]
if ~exist('Seasonality','var')
    taveraging = datenum(datetime(min(year(TPC.Date)),12,1)+calmonths(0:3:12*(max(year(TPC.Date))-min(year(TPC.Date)))));
    T =90;
elseif strcmp(Seasonality,'monthly')==1
    taveraging = datenum(datetime(min(year(TPC.Date)),12,1)+calmonths(0:1:12*(max(year(TPC.Date))-min(year(TPC.Date)))));
    T =31;
elseif strcmp(Seasonality,'three-monthly')==1
    taveraging = datenum(datetime(min(year(TPC.Date)),12,1)+calmonths(0:3:12*(max(year(TPC.Date))-min(year(TPC.Date)))));
    T =90;
elseif strcmp(Seasonality,'annual')==1
    taveraging = datenum(min(year(TPC.Date)):max(year(TPC.Date)),1,1);
    T =365;
end
%%

L = L_timeaverage(TPC.Date,TPC.Distance,taveraging);
D_Error = DE_timeaverage(Discharge,taveraging);

deltaL= 30/1000;
Rho = 0.917;
BTError= BedrockTopoError(Glacier_Centerline);
deltaHmax = sqrt((6.008)^2 + (BTError)^2)/1000;
T = 90;
Hmax=max(MeanH)/1000;
W = MFW/1000;

% Error calculation using the shoelace approach assuming that V1-V2 results 
% in a cuboid polygon.

TMC_Error = zeros(length(L.L),1);
for i=1:length(L.L)
    TMC_Error(i,1) = (Rho/T)*sqrt(((W^2)*(L.L(i)^2)*(deltaHmax^2)) + ((W^2)*(Hmax^2)*(deltaL^2)));
end
Error.Rho = Rho;
Error.T=T;
Error.W= W;
Error.MeanBTError = BTError;
Error.MeanHmax = Hmax;
Error.DeltaHmax=deltaHmax;
Error.DeltaL=deltaL;
Error.L=L;
Error.Delta_D = D_Error;
Error.Delta_TMC = TMC_Error;

end