function [TerminiOutFolder, MeanFjordWith] = TerminiQualityControl(TerminiPath,VelData,CL,UpperPolygon, ...
    LowerPolygon,OutDataFolder,k)
% Function checks initial TermPicks data set (Goliber et al., 2022) and
% removes Delineations that have more than 1 line segment, those that are 
% smaller than Terminus width*k or longer than mean fjord width +2 Sigma.
% It further filters out wrongly geolocated traces, checks that all 
% dates are correct and limits the terminus position to the availability of
% ice discharge data from Mankoff (2019). 
% The function  splits the TermPicks dataset into individual shapefiles 
% based on date, which are saved as they are needed for further processing.
%
% arguments: (input)
% TerminiPath   - Path to directory glacier specific TermPicks dataset.
% UpperPolygon  - Upper fjord wall polygon used to determine Terminus width.
% LowerPolygon  - Lower fjord wall polygon used to determine Terminus width.
% k             - Divider for limiting terminus positions to Terminus width/k (default=2)
%
%
% arguments: (output) 
% TerminiOutFolder - Specifies the path to the relevant split terminus positions
%                    (either all or monthly )

FileName = strsplit(TerminiPath,'/');
GlacierName = FileName{3}(1:end-4);
DataFolder = strcat('Data/Split/',GlacierName,'/');
% Save the quality controlled Terminus positions to a shapefile
TerminusQCFolder = strcat(OutDataFolder,'/TerminusPositions_QC_Av/');


% Delete previous files if the folder already contains files to make sure
% that you are working with the most recent data (avoids duplicates and
% faulty terminus positions to be included in further processing).
disp('Checking Terminus Position Delineation Quality')

if ~exist(DataFolder)
    mkdir(DataFolder)
else
    deletePreviousFiles(DataFolder)
end
if ~exist(TerminusQCFolder)
    mkdir(TerminusQCFolder)
else
    deletePreviousFiles(TerminusQCFolder)
end

Termini = shaperead(TerminiPath);
N=length(Termini);
%% Correct dates
% Some terminus position observation dates are missing a leading zero or have 
% no month/day.The code below adds a leading zero and sets month/day to 1/1/ 
% if none are assigned.
for i=1:N
    Termini(i).Date = strcat(Termini(i).Year,'-',Termini(i).Month,'-',Termini(i).Day);
    if contains(Termini(i).Date,'00-00')==1   
        fprintf('Terminus observation has no month/day: %s \n',Termini(i).Date)
        Termini(i).Date =strrep(Termini(i).Date,'00-00','01-01');
    end
    Termini(i).DateNum = datenum(Termini(i).Date);
    [~, idx] = unique([Termini.DateNum].', 'stable');  
    DateCorrTermini= Termini(idx);
end
%% Limit Terminus Position to Discharge data availability (1986-)
DischargeCutoff = datenum(1986,1,1);
for i=1:length(DateCorrTermini)
    if DateCorrTermini(i).DateNum>DischargeCutoff==1
        DisLimitTermini(i)=DateCorrTermini(i);
    end
end
AllTermini = DisLimitTermini(all(~cellfun(@isempty,struct2cell(DisLimitTermini))));
%%
for i=1:length(AllTermini)
    AllTermini(i).DateNum = datenum(AllTermini(i).Date);
    AllTermini(i).Year = year(AllTermini(i).Date);
    AllTermini(i).Month = month(AllTermini(i).Date);
    AllTermini(i).Day = day(AllTermini(i).Date);
end
%% Sort Termini by date
% For further processing, terminus positions need to be in the correct
% temporal order. 
Afields = fieldnames(AllTermini);
idx = find(contains(Afields,'DateNum'));
Acell = struct2cell(AllTermini);
sz = size(Acell);            
Acell = reshape(Acell, sz(1), []);     
Acell = Acell';                        
Acell = sortrows(Acell,idx); 
Acell = reshape(Acell', sz);
SortedTermini = cell2struct(Acell, Afields, 1);

%% Remove Delineations that have more than 1 line segment 
% Some delineations are 2 point connections that are seperated by a NaN. If
% we would remove the NaN values, we would assume the terminus position at
% that location, which could lead to errors. Therefore we exclude 
% delineations with more than 1 line segment excluded here.
for i=1:length(AllTermini)
    TerminusNodes=[AllTermini(i).X;AllTermini(i).Y]';
    NumNaNs = sum(sum(isnan(TerminusNodes)));
    NaNLoc = find(isnan(TerminusNodes));
    if NumNaNs<4 && length(NaNLoc)<=2
        AllTerminiNew(i)=AllTermini(i);
    elseif length(NaNLoc)>2  
        AllTerminiNew(i) = AllTermini(i);
        AllTerminiNew(i).X = AllTermini(i).X(~isnan(AllTermini(i).X));
        AllTerminiNew(i).Y = AllTermini(i).Y(~isnan(AllTermini(i).Y));
    end
end
TerminiCleaned1=AllTerminiNew(all(~cellfun(@isempty,struct2cell(AllTerminiNew))));
 %% Filter Delineations by length relative to terminus width
 % Removes terminus positions that are shorter than the fjord width*k or
 % longer than fjord width +2 sigma

[~, FjordWidth] = distance2curve(UpperPolygon.Vertices,LowerPolygon.Vertices);
FilteredTermini3 = TerminiCleaned1;
for m=1:length(TerminiCleaned1)
    TerminusNodesNew=[TerminiCleaned1(m).X;TerminiCleaned1(m).Y]';
    TerminusVertices=TerminusNodesNew(all(~isnan(TerminusNodesNew),2),:);
    d = diff(TerminusVertices); 
    Terminus_length(m,1) = sum(sqrt(sum(d.*d,2)));
    a = fields(FilteredTermini3);
    if strcmp(GlacierName,'Sermeq_Avanarleq')==1
        if Terminus_length(m,1)>=round(mean(FjordWidth),-3)*k
             FilteredTermini3(m)=TerminiCleaned1(m);
        else
            for g = 1:length(a)
                FilteredTermini3(m).(a{g})=[];
            end
        end
        if Terminus_length(m,1)>(mean(FjordWidth)+std(FjordWidth)*2)==1
            for g = 1:length(a)
                FilteredTermini3(m).(a{g})=[];
            end
        end   
    else 
        if Terminus_length(m,1)>=min(FjordWidth)*k==1  
                FilteredTermini3(m)=TerminiCleaned1(m);
        elseif Terminus_length(m,1)>(nanmean(FjordWidth)+nanstd(FjordWidth)*2)==1
            for g = 1:length(a)
                FilteredTermini3(m).(a{g})=[];
            end
        else
            for g = 1:length(a)
                FilteredTermini3(m).(a{g})=[];
            end
        end
    end   
end


% % Remove all empty entries in sturcture to create data set
TerminiCleaned2 = FilteredTermini3(all(~cellfun(@isempty,struct2cell(FilteredTermini3))));
for i=1:length(TerminiCleaned2)
    TerminiCleaned2(i).Datenum = datenum(TerminiCleaned2(i).Date);
    TerminiCleaned2(i).X = TerminiCleaned2(i).X(:,~isnan(TerminiCleaned2(i).X(:))) ;
    TerminiCleaned2(i).Y = TerminiCleaned2(i).Y(:,~isnan(TerminiCleaned2(i).Y(:))) ;
end
%% Velocity Filter
% In some instances, delineations show large advances or retreats of the
% glacier, which appear to be unrealistic. Using ITS LIVE composite
% velocities, we check if the distance between terminus positions fits
% within the observed velocities. If not, the terminus position is removed
% from the dataset. 

for c=1:length(TerminiCleaned2)-1
    CL_data = shaperead(CL);
    [xi,yi]= polyxpoly(CL_data.X,CL_data.Y,TerminiCleaned2(c).X,TerminiCleaned2(c).Y);
    [xj,yj]= polyxpoly(CL_data.X,CL_data.Y,TerminiCleaned2(c+1).X,TerminiCleaned2(c+1).Y);
    if ~isempty(xi) && ~isempty(xj)
        distance(c,1) = sqrt((xj(1)-xi(1))^2 + (yj(1)-yi(1))^2);
    else
        [xi,yi] = deal(0);
        [xj,yj] = deal(0);
        distance(c,1) = 0;
    end
    % Get Normal to determine line between two terminus position center points. 
    A = [xi(1),yi(1)];
    B = [xj(1),yj(1)];
    V = B-A;
    midV = A+0.5*V;
    normal1 = [V(2),-V(1)];
    normal2 = [-V(2),V(1)];
    
    NormalX = [(midV(1) + normal1(1)),(midV(1) + normal2(1))];
    NormalY = [(midV(2) + normal1(2)),(midV(2) + normal2(2))];
    
    m = (NormalY(2) - NormalY(1)) / (NormalX(2) - NormalX(1));
    b = NormalY(1) - m * NormalX(1);
    
    if any(TerminiCleaned1(c).X(1:end-1)<0)==1
        leftOfLine = xj < (yj - b) / m;
        if leftOfLine==1
            fprintf('Advance \n');
        else
            fprintf('Retreat \n');
            distance(c,1) = distance(c,1)*(-1);
        end
    else
        leftOfLine = xj < (yj - b) / m;
        if leftOfLine==1 
            fprintf('Retreat \n');
            distance(c,1) = distance(c,1)*(-1);
        else
            fprintf('Advance \n');
        end
    end

    %Determine observation years for subsequent terminus positions
    StartYear = year(TerminiCleaned1(c).DateNum);
    EndYear = year(TerminiCleaned1(c+1).DateNum);
    TP_TimeDiff(c,1) = days(datetime(TerminiCleaned2(c+1).DateNum,'ConvertFrom','datenum')-datetime(TerminiCleaned2(c).DateNum,'ConvertFrom','datenum'));
    % Read in Velocity data & calculate average velocity for period
    % 2018-2018
    VelMat = readmatrix(VelData);

    NewTerminiCleaned(c)=TerminiCleaned2(c);
    VelMean = mean(VelMat(end-10:end,2));
    

    if abs(distance(c,1))>2*VelMean 
        disp('Terminus Change greater than Velocity')
        NewTerminiCleaned(c)=[];
    else
        disp('Terminus Change within Velocity boundaries')
        NewTerminiCleaned(c) = TerminiCleaned2(c);
    end
        TerminiVFiltered = NewTerminiCleaned(all(~cellfun(@isempty,struct2cell(NewTerminiCleaned))));
    end
% end

%% Remove all empty entries in sturctureand write the dataset to file
FinalDataSet = TerminiVFiltered(all(~cellfun(@isempty,struct2cell(TerminiVFiltered))));
shapewrite(FinalDataSet,strcat(TerminusQCFolder,GlacierName,'.shp'))

%% Write individual shapefiles for further processing
for i=1:length(FinalDataSet)
    data = FinalDataSet(i);
    Name = datestr(datetime(FinalDataSet(i).Date));
    OutDir = dir([DataFolder,'*.shp']);
    FileNo = numel(OutDir);
    if FileNo~=length(AllTermini)
        shapewrite(data,strcat(DataFolder,Name,'.shp'))
    end
end
TerminiOutFolder=DataFolder;
MeanFjordWith = nanmean(FjordWidth);
