function AnnualAverageFolder = VelocityExtractionLine(VelNetCDF,CLPath,TP_Path)
% Function extracts ice flow velocity from NASA ITS_LIVE composite
% velocity maps which have been downloaded previously. The velocities are
% extracted along a centerline, which is constructed between the midpoints
% of the most advanced and the most retreated terminus position. The data
% is then averaged annually and written to file. 
%
% 
% arguments: (input)
% VelNetCDF - Path to the previously downloaded NASA ITS_LIVE velocity maps
% CLPath - Path to the centeline associated with the AOI
% TP_Path - Path to the terminus delineations of the glacier
%
% arguments: (output)
% AnnualAverageFolder - Path to annual mean velocity file

disp('Extracting Velocities')
TP = shaperead(fullfile(TP_Path));
CL = shaperead(CLPath);
V_files = dir(fullfile(VelNetCDF,'*.nc'));
V_Names = {V_files.name};
Dates = V_Names;
[~,filename,~]=fileparts(TP_Path);
OutFolder = 'Data/Velocity/Annual_Averages_Line/';
%%

%% Get intersection point between observations and Centerline
for i=1:length(TP)
    Coords{i} = [TP(i).X', TP(i).Y'];
    [xn{i},yn{i}] = polyxpoly(TP(i).X,TP(i).Y,CL.X,CL.Y);
end
%% Clean up Structures and Arrays
Empty = find(~cellfun(@isempty,xn));
[~,index] = sortrows({TP.Date}.');
TP = TP(index); 
TP_C = TP(Empty);

for i=1:length(Empty)
    NewCoordsX(i,1) = xn{Empty(i)}(1);
    NewCoordsY(i,1) = yn{Empty(i)}(1);
end
%% Get distance between intersection points
for j=1:length(TP_C)
    for k=1:length(TP_C)-1
        P1x = NewCoordsX(j);
        P1y = NewCoordsY(j);
        P2x = NewCoordsX(end-k);
        P2y = NewCoordsY(end-k);
        distance(j,1) = sqrt((P2x(1)-P1x(1))^2 + (P2y(1)-P1y(1))^2);
    end
end

%% Create Extraction line from longest distance between intersections
% [~,idx] = max(distance);
% if (idx~=length(TP_C))==1
%     ExtractionPoint(1,:) = [NewCoordsX(idx);NewCoordsY(idx)];
%     ExtractionPoint(2,:) = [NewCoordsX(end-idx);NewCoordsY(end-idx)];
%     ExtractionLine = [linspace(ExtractionPoint(1,1),ExtractionPoint(2,1),100);linspace(ExtractionPoint(1,2),ExtractionPoint(2,2),100)]'; 
% else
%     ExtractionPoint(1,:) = [NewCoordsX(idx);NewCoordsY(idx)];
%     ExtractionPoint(2,:) = [NewCoordsX(end-(idx-1));NewCoordsY(end-(idx-1))];
%     ExtractionLine = [linspace(ExtractionPoint(1,1),ExtractionPoint(2,1),100);linspace(ExtractionPoint(1,2),ExtractionPoint(2,2),100)]'; 
% end
%% Extract data along line from ITS_LIVE DATA and save to file
OutMat = zeros(length(V_Names),2);
for i=1:length(V_Names)
    data = ncread(fullfile(V_files(1).folder,V_Names{i}),'v');
    x = ncread(fullfile(V_files(1).folder,V_Names{i}),'x');
    y = ncread(fullfile(V_files(1).folder,V_Names{i}),'y');
    data(data==-32767)=NaN;
    
    [netx,nety] = meshgrid(x,y);

    v{i} = improfile(netx,nety,flipud(data'),NewCoordsX,NewCoordsY);
end

for j=1:length(v)
    OutMat(j,1) = str2double(Dates{j}(11:end-3));
    OutMat(j,2)=nanmean(v{j});
end
    if ~exist(OutFolder)
        mkdir(OutFolder)
    end
        writematrix(OutMat,strcat(OutFolder,filename,'_Line','.csv'));
        AnnualAverageFolder = strcat(OutFolder,filename,'_Line','.csv'); 
end
% end
