function plotFrontalAblation(Discharge,MassData,TPChange,Velocity,tePos,TMC_Error,...
    GlacierName,Seasonality,OutFolder)
% Function calculates time averaged frontal ablation and plots results
%
% arguments (input):
% Discharge     - Path to ice discharge data
% MassData      - Matrix containing previously determine mass
% TPChange      - Terminus positions change
% Velocity      - Glacier velocities extracted along centerline
% tePos         - Terminus positions used in the calculation
% Error         - Previously calculated uncertainties
% GlacierName   - Name of the glacier
% Seasonality   - Desired seasonality (default:3-monthly)
% OutFolder     - Folder for saving resulting figure

    a = readtable(Discharge);
    tD = datenum(table2array(a(:,2)));
    D0 = table2array(a(:,3));
    % remove duplicates
    inds = find(diff(tD)==0);
    tD(inds) = [];
    D0(inds) = [];

    % make units Gt/day
    D0 = D0/365.25;

    tM = MassData(:,1);
    M0 = MassData(:,5);

% Check which seasonality is requested to be plotted (Default =
% 3-monthly)
    if ~exist('Seasonality','var')
        taveraging = datenum(datetime(min(year(MassData(:,1))),12,1)+calmonths(0:3:12*(max(year(MassData(:,1)))-min(year(MassData(:,1))))));
    elseif strcmp(Seasonality,'monthly')==1
        taveraging = datenum(datetime(min(year(MassData(:,1))),12,1)+calmonths(0:1:12*(max(year(MassData(:,1)))-min(year(MassData(:,1))))));
        Ndays =31;
    elseif strcmp(Seasonality,'three-monthly')==1
        taveraging = datenum(datetime(min(year(MassData(:,1))),12,1)+calmonths(0:3:12*(max(year(MassData(:,1)))-min(year(MassData(:,1))))));
        Ndays =90;
    elseif strcmp(Seasonality,'annual')==1
        taveraging = datenum(min(year(MassData(:,1))):max(year(MassData(:,1))),1,1);
        Ndays =365;
    end
if sum(~isnan(D0))>0
%% Time average mass and discharge data and calculate frontal ablation
    s = timeaverage(tM,M0,tD,D0,taveraging,TMC_Error,OutFolder,GlacierName);
%% Plot results
    nIDs = 5;
    alphabet = ('A':'Z').';
    chars = num2cell(alphabet(1:nIDs));
    chars = chars.';
    charlbl = strcat('[',chars,']'); 
    DateMin = min(MassData(:,1));%arrayfun(@(s) min(s.tbounds), s);
    DateMax = max(MassData(:,1));%arrayfun(@(s) max(s.tbounds), s);
    ms = 2;
    lw1 = 0.25;
    lw2 = 1;
    % 
    f2=figure(2);
    f2.WindowState = 'maximized';
    
    ax2=subplot(5,2,1);
    plot(Velocity(:,1),Velocity(:,2),'k','LineWidth',lw2)
    datetick('x','yyyy','keeplimits');
    xlim([DateMin DateMax])
    ylabel('Velocity [m/y]')
    grid on
    text(.01,0.89,charlbl{1},'Units','normalized','FontSize',16)
    set(gca,'fontsize',12)

    
    ax3 = subplot(5,2,3);
    plot(TPChange.Date,TPChange.Distance,'color','#702963', 'LineWidth',lw2);
    datetick('x','yyyy','keeplimits');
    xlim([DateMin DateMax]);
    ylabel({'TP relative to'; 'most recent obs. [m]'});
    grid on
    text(.01,0.89,charlbl{2},'Units','normalized','FontSize',16)
    set(gca,'fontsize',12)

    ax4=subplot(5,2,[5 7]);
    cols = winter(length(MassData));
    cmap = colormap(cols);
    % 
    hold on
    p(1) = plot(s.t,s.D,'s-','markersize',ms,'linewidth',lw1);
    p(2) = plot(s.t,s.TMC,'s-','markersize',ms,'linewidth',lw1);
    p(3) = plot(s.t,s.FA,'s-','markersize',ms,'linewidth',lw1);
    for i=1:length(s.t)
        plot([s.tbounds(i),s.tbounds(i+1)],s.D(i)*[1,1],'color',p(1).Color,'linewidth',lw2);
        plot([s.tbounds(i),s.tbounds(i+1)],s.TMC(i)*[1,1],'color',p(2).Color,'linewidth',lw2);
        plot([s.tbounds(i),s.tbounds(i+1)],s.FA(i)*[1,1],'color',p(3).Color,'linewidth',lw2);      
    end

    set(gca,'box','on'); grid on;
    datetick('x','yyyy','keeplimits');
    xlim([DateMin DateMax]);
    ylabel('mass flux (Gt/d)');
    l1 = legend('D','TMC','F','NumColumns',3);
    MeanE = num2str(mean(s.Error));
    annotation(f2,'textbox',...
    [0.151520833333333 0.549325026486644 0.0786458311447252 0.0280373826514647],...
    'String',{['Max Error: ',MeanE];},...
    'EdgeColor','None');
    text(.01,0.95,charlbl{3},'Units','normalized','FontSize',16)
    set(gca,'fontsize',12)
    hold off

    ax5 = subplot(5,2,9);
    for k=1:length(MassData)
        bar(MassData(k,1),MassData(k,5),'FaceColor',cmap(k,:),'EdgeColor','None','BarWidth',20);
        hold on
    end
    hold off
    datetick('x','yyyy','keeplimits')
    xlim([DateMin DateMax]);
    set(gca,'TickDir','out')
    set(gca,'box','on');grid on;
    ylabel('Mass [Gt]')
    xlabel('Year')
    text(.01,0.93,charlbl{4},'Units','normalized','FontSize',16)
    set(gca,'fontsize',12)

    subplot(5,2,[2 10])
    for g=1:length(MassData)
        X = tePos{g}.X;
        Y = tePos{g}.Y;
        [lat{g},lon{g}] = polarstereo_inv(X,Y,[],[],70,-45);
        p=geoplot(lat{g},lon{g});
        geobasemap satellite;
        p.Color = cmap(g,:);
        hold on
    end
    grid off
    LabelDates =unique(year(MassData(:,1)));
    cbr =colorbar;
    cbr.Ticks = linspace(0,1,10);
    cbr.TickLabels = round(linspace(min(LabelDates),max(LabelDates),10));
    txt=text(.01,0.97,charlbl{5},'Units','normalized','FontSize',16);
    set(txt,'BackgroundColor',[1 1 1]);
    clear latlon
    set(gca,'fontsize',12)
    SaveFolderImage = strcat(OutFolder,'/FA_Img/');
    if ~exist(SaveFolderImage)
        mkdir(SaveFolderImage)
    end
    pause(3)
exportgraphics(gcf,strcat(SaveFolderImage,GlacierName,'.jpg'),'Resolution',400);
clear MinE MaxE 
hold off

close(f2)

else % Plot if no discharge data is available in Mankoff et al. (2019)
    nIDs = 5;
    alphabet = ('A':'Z').';
    chars = num2cell(alphabet(1:nIDs));
    chars = chars.';
    charlbl = strcat('[',chars,']'); 
    DateMin = min(MassData(:,1));
    DateMax = max(MassData(:,1));
    ms = 2;
    lw1 = 0.25;
    lw2 = 1;
    
    f4=figure(4);
    f4.WindowState = 'maximized';
    
    ax2=subplot(5,2,1);
    plot(Velocity(:,1),Velocity(:,2),'k','LineWidth',lw2)
    datetick('x','yyyy','keeplimits');
    xlim([DateMin DateMax])
    ylabel('Velocity [m/y]')
    grid on
    text(.01,0.89,charlbl{1},'Units','normalized','FontSize',16)
    set(gca,'fontsize',12)

    ax3 = subplot(5,2,3);
    plot(TPChange.Date,TPChange.Distance,'color','#702963', 'LineWidth',lw2);
    datetick('x','yyyy','keeplimits');
    xlim([DateMin DateMax]);
    ylabel({'TP relative to'; 'most recent obs. [m]'});
    grid on
    text(.01,0.89,charlbl{2},'Units','normalized','FontSize',16)
    set(gca,'fontsize',12)

    % 
    ax4=subplot(5,2,[5 7]);
    cols = winter(length(MassData));
    cmap = colormap(cols);
    % 
    plot(MassData(:,1),MassData(:,5),'k','LineWidth',lw2)
    set(gca,'box','on'); grid on;
    datetick('x','yyyy','keeplimits');
    xlim([DateMin DateMax]);
    ylabel('Volume (Gt)');
    text(.01,0.93,charlbl{3},'Units','normalized','FontSize',16)
    set(gca,'fontsize',12)
    
    ax5 = subplot(5,2,9);
    for k=1:length(MassData)
        bar(MassData(k,1),MassData(k,5),'FaceColor',cmap(k,:),'EdgeColor','None','BarWidth',20);
        hold on
    end
    datetick('x','yyyy','keeplimits')
    xlim([DateMin DateMax]);
    set(gca,'TickDir','out')
    set(gca,'box','on');grid on;
    ylabel('Mass [Gt]')
    xlabel('Year')
    text(.01,0.93,charlbl{4},'Units','normalized','FontSize',16)
    set(gca,'fontsize',12)

    subplot(5,2,[2 10])
    for g=1:length(MassData)
        X = tePos{g}.X;
        Y = tePos{g}.Y;
        [lat{g},lon{g}] = polarstereo_inv(X,Y,[],[],70,-45);
        p=geoplot(lat{g},lon{g});
        geobasemap satellite;
        p.Color = cmap(g,:);
        hold on
    end
    grid off
    LabelDates =unique(year(MassData(:,1)));
    cbr =colorbar;
    cbr.Ticks = linspace(0,1,10);
    cbr.TickLabels = round(linspace(min(LabelDates),max(LabelDates),10));
    txt=text(.01,0.97,charlbl{5},'Units','normalized','FontSize',16);
    set(txt,'BackgroundColor',[1 1 1]);
    clear latlon
    hold off
    set(gca,'fontsize',12)
    
    SaveFolderImage = strcat(OutFolder,'/FA_Img/');
    if ~exist(SaveFolderImage)                                          
        mkdir(SaveFolderImage)
    end

    exportgraphics(gcf,strcat(SaveFolderImage,GlacierName,'_VolumeOnly.jpg'),'Resolution',400);
end