function s = timeaverage(tM,M0,tD,D0,t,TMC_Error,OutFolderData,GlacierName)

% function to create average frontal ablation from sparse
% and misaligned inputs

% % arguments: (input)
% tM            - time of terminus mass as a matlab datenum
% M0            - mass of terminus
% tD            - time of solid ice discharge as a matlab datenum
% D0            - solid ice discharge
% t             - intervals of time on which to output data i.e. data is 
%                 averaged between t(i+1) and t(i)
% OutFolderData - Desired output folder
% GlacierName   - Name of glacier

% % arguments: (output) 
% s.tbounds - boundaries of time intervals on which output data defined
% s.t       - midpoint of time intervals on which output data defined
% s.D       - discharge during time intervals
% s.TMC     - terminus mass change during time intervals
% s.F       - frontal ablation during time intervals
% s.D0      - discharge masking out interpolated values
% s.TMC0    - terminus mass change masking out interpolated values
% s.F0      - frontal ablation masking out interpolated values

%% terminus mass change

% interpolate M onto t
M = interp1(tM,M0,t,'linear',NaN);

% mass change during time period in units Gt/d
dMdt = diff(M)./diff(t);


%% discharge

% interpolate discharge to daily
tDdaily = [min(t):max(t)];
Ddaily = interp1(tD,D0,tDdaily,'linear',NaN);

% mean values over required intervals
for i=1:length(t)-1
    inds = find(tDdaily>=t(i) & tDdaily<t(i+1));
    D(i) = mean(Ddaily(inds));
end
%% Discharge Error
% 
% interpolate discharge to daily
% tDErrordaily = [min(t):max(t)];
% DErrordaily = interp1(tD,D0Error,tDErrordaily,'linear',NaN);
% 
% % mean values over required intervals
% for i=1:length(t)-1
%     inds = find(tDErrordaily>=t(i) & tDErrordaily<t(i+1));
%     DError(i) = mean(DErrordaily(inds));
% end

%% Total Error
Max_Error = max(max(sqrt((TMC_Error.Delta_D.D_error).^2 + (TMC_Error.Delta_TMC).^2)));

%% combine into frontal ablation

F = D-dMdt;

%% create version of data with no interpolated values

% mask M for values that are pure interpolation
% for an interpolated value to be reliable need values either side
inds = [];
for i=2:length(t)-1
    if or(isempty(find(tM>=t(i-1) & tM<t(i))),...
          isempty(find(tM>=t(i) & tM<t(i+1))))
         inds = [inds,i];
    end
end

% mass change during time period removing interpolated values
M0 = M; M0(inds) = NaN;
dMdt0 = diff(M0)./diff(t);

% check if there is a D value during the time periods
inds = [];
for i=2:length(t)-1
    if isempty(find(tD>=t(i) & tD<t(i+1)))
         inds = [inds,i];
    end
end

% remove discharge values that are interpolated
D0 = D; D0(inds) = NaN;

% calculate frontal ablation
F0 = D0-dMdt0;

%% put into output structure

s.tbounds = t;
s.t = 0.5*(t(1:end-1)+t(2:end));
s.D = D;
s.TMC = dMdt;
s.FA = F;
s.D0 = D0;
s.TMC0 = dMdt0;
s.FA0 = F0;
s.Error = Max_Error;
SaveFolder = strcat(OutFolderData,'/FA_Mat/');
if ~exist(SaveFolder)
    mkdir(SaveFolder)
end
save(strcat(SaveFolder,GlacierName,'.mat'),'s')
end
